/*
 * Copyright 2013 Yuichiro Moriguchi
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package net.morilib.awk.nano.builtin;

import java.math.BigDecimal;
import java.util.Map;

import net.morilib.awk.nano.io.AwkFiles;
import net.morilib.awk.nano.namespace.AwkNamespace;
import net.morilib.awk.nano.value.AwkArray;
import net.morilib.awk.nano.value.AwkInteger;
import net.morilib.awk.nano.value.AwkValue;

/**
 * 組み込み関数intの記述です。
 *
 *
 * @author MORIGUCHI, Yuichiro 2013/04/13
 */
public class AwkInt extends AwkUnaryArgs {

	/**
	 * 関数を生成します。
	 */
	protected AwkInt() {
		super("int");
	}

	@Override
	public AwkValue apply(AwkNamespace ns, AwkFiles o, AwkValue a1) {
		AwkArray a;
		double x;

		if(a1.isArray()) {
			a = new AwkArray();
			for(Map.Entry<String, AwkValue> z : a1.entrySet()) {
				a.putArray(z.getKey(), apply(ns, o, z.getValue()));
			}
			return a;
		} else if(a1.isInteger()) {
			return a1;
		} else {
			x = a1.toFloat();
			x = x > 0 ? Math.floor(x) : Math.ceil(x);
			return AwkInteger.valueOf(
					new BigDecimal(x).toBigInteger());
		}
	}

}
