/*
 * Copyright 2013 Yuichiro Moriguchi
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package net.morilib.awk.nano.builtin;

import java.util.Map;

import net.morilib.awk.nano.io.AwkFiles;
import net.morilib.awk.nano.namespace.AwkNamespace;
import net.morilib.awk.nano.value.AwkArray;
import net.morilib.awk.nano.value.AwkValue;

/**
 * awkiumの数学関数の基底クラスです。<br />
 * 配列についても適用できるよう拡張してあります。
 *
 *
 * @author MORIGUCHI, Yuichiro 2013/04/13
 */
public abstract class AwkMathFunction extends AwkUnaryArgs {

	/**
	 * 関数を生成します。
	 * 
	 * @param s 関数名
	 */
	protected AwkMathFunction(String s) {
		super(s);
	}

	/**
	 * 関数を計算します。
	 * 
	 * @param x 値
	 * @return 結果
	 */
	protected abstract AwkValue f(AwkValue x);

	@Override
	public AwkValue apply(AwkNamespace ns, AwkFiles o, AwkValue a1) {
		AwkArray a;

		if(a1.isArray()) {
			a = new AwkArray();
			for(Map.Entry<String, AwkValue> z : a1.entrySet()) {
				a.putArray(z.getKey(), f(z.getValue()));
			}
			return a;
		} else {
			return f(a1);
		}
	}

}
