/*
 * Copyright 2013 Yuichiro Moriguchi
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package net.morilib.awk.nano.pattern;

import net.morilib.awk.nano.expr.AwkExpression;
import net.morilib.awk.nano.io.AwkFiles;
import net.morilib.awk.nano.namespace.AwkNamespace;

/**
 * awkium式によるのパターン中間表現です。
 * 
 * @author MORIGUCHI, Yuichiro 2013/03
 */
public class AwkExpressionPattern implements AwkPattern {

	private AwkExpression expr;

	/**
	 * 中間表現を生成します。
	 * 
	 * @param expr 式の中間表現
	 */
	public AwkExpressionPattern(AwkExpression expr) {
		this.expr = expr;
	}

	public boolean isBegin() {
		return false;
	}

	public boolean isEnd() {
		return false;
	}

	public AwkMatchState match(AwkNamespace ns, AwkFiles fs,
			String s, AwkMatchState stat) {
		return expr.eval(ns, fs).toBoolean(ns) ?
				AwkMatchState.MATCHED : AwkMatchState.NO_MATCH;
	}

	public boolean isRanged() {
		return false;
	}

}
